// force vc7 to group all shader related stuff into the Shader namespace
#ifdef WIN32
namespace Shader {
#endif

// define inputs from application
struct appin
{
  float4 position   : POSITION;
  float3 normal     : NORMAL;
  float2 texCoords0 : TEXCOORD0;
  float2 texCoords1 : TEXCOORD1;
  float3 tangent    : TEXCOORD2;
};

// define outputs from vertex shader
struct vertout
{
  float4 position   : POSITION;
  float2 texCoords0 : TEXCOORD0;
  float2 texCoords1 : TEXCOORD1;
  float2 texCoords2 : TEXCOORD2;
  float3 lightDir   : TEXCOORD3;
  float3 halfAngle  : TEXCOORD4;
  float3 eyeDir     : TEXCOORD5;
};

vertout main(appin v_in,
	     uniform float4x4 MVPMatrix, 
	     uniform float4 lightPosition0,   // object space
	     uniform float4 eyePosition)      // object space
{
  vertout v_out;

  // transform vertex position into homogenous clip-space
  v_out.position   = mul(MVPMatrix, v_in.position);
  v_out.texCoords0 = v_in.texCoords0;
  v_out.texCoords1 = v_in.texCoords1;
  v_out.texCoords2 = v_in.texCoords0;

  float3 binormal = cross(v_in.tangent, v_in.normal);
  float3x3 rotation = float3x3(v_in.tangent, binormal, v_in.normal);
  v_out.lightDir = normalize(mul(rotation, lightPosition0.xyz));

  // Calculate half-angle in tangent space
  float3 eyeDir = eyePosition - v_in.position.xyz;
  float3 halfAngle = normalize(eyeDir + lightPosition0.xyz);//normalize(eyeDir + v_out.lightDir);
  v_out.halfAngle = mul(rotation, halfAngle);
  v_out.eyeDir = normalize(mul(rotation, eyeDir)); 
  //v_out.eyeDir = eyeDir;

  return v_out;
}

#ifdef WIN32
}
#endif
